(***************************************************************)
(*                                                             *)
(*                      camlgrenouille                         *)
(*                                                             *)
(*             Copyright 2001-2005 Alan Schmitt                *)
(*              alan.schmitt@polytechnique.org                 *)
(*                                                             *)
(* This file is distributed under the terms of the GNU General *)
(* Public License.                                             *)
(*                                                             *)
(***************************************************************)

open Debug
open Abort

type usrconf = {
    mutable username : string;
    mutable password : string;
    mutable interface : string;
    mutable ifconfig : string;
    mutable awk : string;
    mutable grep : string;
    mutable cut : string;
    mutable ping : string;
    mutable normal_result_delay : int;
    mutable post_results : bool;
    mutable ping_string : bool;
    mutable passive : bool;
    mutable ratiodl : int;
    mutable ratioul : int;
    mutable noratiodl : bool;
    mutable noratioul : bool;
    mutable testping : bool;
    mutable testdl : bool;
    mutable testul : bool;
    mutable testbk : bool;
    mutable dlhost : string;
    mutable dlpath : string;
    mutable dlfile : string;
    mutable ulhost : string;
    mutable ulpath : string;
    mutable ulfile : string;
    mutable dl_out : string;
    mutable ul_out : string;
    mutable ping_out : string;
    mutable breakdown_out : string;
    mutable ftp_password : string;
    mutable test_ip : bool;
    mutable plugin : string;
    mutable plugin_password : string;
    mutable plugin_username : string;
    mutable plugin_port : string;
    mutable grenouille_file : string;
    mutable grenouille_server : string;
    mutable grenouille_port : int;
    mutable alt_grenouille_file : string;
    mutable alt_grenouille_server : string;
    mutable alt_grenouille_port : int;
    mutable detect_router : bool;
} 

let usrconf = {
  username = "";
  password = "";
  interface = "ppp0";
  ifconfig = "/sbin/ifconfig";
  awk = "/bin/awk";
  grep = "/bin/grep";
  cut = "/usr/bin/cut";
  ping = "/bin/ping";
  normal_result_delay = 1800;
  post_results = true;
  ping_string = false;
  passive = true;
  ratiodl = 0;
  ratioul = 0;
  noratiodl = false;
  noratioul = false;
  testping = true;
  testdl = true;
  testul = true;
  testbk = true;
  dlhost = "";
  dlpath = "";
  dlfile = "";
  ulhost = "";
  ulpath = "";
  ulfile = "";
  dl_out = "";
  ul_out = "";
  ping_out = "";
  breakdown_out = "";
  ftp_password = "";
  test_ip = false;
  plugin = "";
  plugin_password = "";
  plugin_username = "";
  plugin_port = "";
  grenouille_file = "/interface.php";
  grenouille_server = "interface.grenouille.com";
  grenouille_port = 81;
  alt_grenouille_file = "/interface.php";
  alt_grenouille_server = "";
  alt_grenouille_port = 80;
  detect_router = false;
}

exception Finished of int

let exec_out cmd args =
  match cmd with
  | "" -> 0
  | _ -> Sys.command (cmd ^ " " ^ args)
	
let remove_trailing_spaces s =
  try
    for i = (String.length s) - 1 downto 0 do
      if (String.get s i <> ' ') && (String.get s i <> '\r') then raise (Finished i)
    done;
    ""
  with
  | Finished i -> String.sub s 0 (i + 1)
  | e -> raise e

let parse_config input_file =
  let inp = open_in input_file in
  try
    while true do
      let l = input_line inp in
      if (String.length l) <> 0 then
	if (String.get l 0) <> '#' then
	  try
	    let sp = String.index l ' ' in
	    let param = String.sub l 0 sp in
	    let p_val = remove_trailing_spaces (String.sub l (sp + 1) ((String.length l) - sp - 1)) in
	    dprint Debug ("["^param^"] = ["^p_val^"]");
	    match param with
	    | "username" -> usrconf.username <- p_val
	    | "password" -> usrconf.password <- p_val
	    | "interface" -> usrconf.interface <- p_val
	    | "ifconfig"-> usrconf.ifconfig <- p_val
	    | "awk" -> usrconf.awk <- p_val
	    | "grep" -> usrconf.grep <- p_val
	    | "cut" -> usrconf.cut <- p_val
	    | "ping" -> usrconf.ping <- p_val
	    | "normal_result_delay" -> usrconf.normal_result_delay <- (int_of_string p_val)
	    | "missed_result_delay" -> dprint Normal "The parameter missed_result_delay is no longer supported"
	    | "missed_config_delay" -> dprint Normal "The parameter missed_config_delay is no longer supported"
	    | "post_results" -> usrconf.post_results <- (bool_of_string p_val)	    	    
	    | "ping_string" -> dprint Normal "The parameter ping_string is no longer supported"
	    | "passive" -> usrconf.passive <- (p_val = "true")
	    | "ratiodl" -> usrconf.ratiodl <- (int_of_string p_val)
	    | "ratioul" -> usrconf.ratioul <- (int_of_string p_val)
	    | "noratiodl" -> usrconf.noratiodl <- (p_val = "true")
	    | "noratioul" -> usrconf.noratioul <- (p_val = "true")
	    | "testping" -> usrconf.testping <- (bool_of_string p_val)
	    | "testdl" -> usrconf.testdl <- (bool_of_string p_val)
	    | "testul" -> usrconf.testul <- (bool_of_string p_val)
	    | "testbk" -> usrconf.testbk <- (p_val = "true")
	    | "dlhost" -> usrconf.dlhost <- p_val
	    | "dlpath" -> usrconf.dlpath <- p_val
	    | "dlfile" -> usrconf.dlfile <- p_val
	    | "ulhost" -> usrconf.ulhost <- p_val
	    | "ulpath" -> usrconf.ulpath <- p_val
	    | "ulfile" -> usrconf.ulfile <- p_val
	    | "dl_cmd" -> usrconf.dl_out <- p_val
	    | "ul_cmd" -> usrconf.ul_out <- p_val
	    | "ping_cmd" -> usrconf.ping_out <- p_val
	    | "breakdown_cmd" -> usrconf.breakdown_out <- p_val
	    | "ftp_password" -> usrconf.ftp_password <- p_val
	    | "plugin" -> usrconf.plugin <- p_val
 	    | "plugin_password" -> usrconf.plugin_password <- p_val
	    | "plugin_username" -> usrconf.plugin_username <- p_val
	    | "plugin_port" -> usrconf.plugin_port <- p_val
            | "grenouille_file" -> usrconf.grenouille_file <- p_val
            | "grenouille_server" -> usrconf.grenouille_server <- p_val
            | "grenouille_port" -> usrconf.grenouille_port <- (int_of_string p_val)
            | "alt_grenouille_file" -> usrconf.alt_grenouille_file <- p_val
            | "alt_grenouille_server" -> usrconf.alt_grenouille_server <- p_val
            | "alt_grenouille_port" -> usrconf.alt_grenouille_port <- (int_of_string p_val)
	    | "detect_router" -> usrconf.detect_router <- (bool_of_string p_val)
	    | s -> dprint Normal ("The following parameter is unknown: "^s)
	  with 
	  | Not_found -> dprint Debug "Not_found exception raised"   
    done;
    true
  with
  | End_of_file -> myign close_in inp; true
  | e ->
      myign close_in inp;
      dprint Normal ("problem reading configuration file "^input_file^":\n" ^ (Printexc.to_string e));
      false
