(***************************************************************)
(*                                                             *)
(*                      camlgrenouille                         *)
(*                                                             *)
(*             Copyright 2001-2005 Alan Schmitt                *)
(*              alan.schmitt@polytechnique.org                 *)
(*                                                             *)
(* This file is distributed under the terms of the GNU General *)
(* Public License.                                             *)
(*                                                             *)
(***************************************************************)

open Debug
open Unix
open Abort

let timeout_time = 20.0

let rec parse_stuff s =
  try
    let pos = String.index !s '\n' in
    (* first \r\n *)
    let line = String.sub !s 0 (pos) in
    let line' =
      if (pos <> 0) then
	if String.get line (pos-1) = '\r' then
	  String.sub line 0 (pos-1)
	else line
      else line
    in
    s := String.sub !s (pos + 1) ((String.length !s) - pos - 1);
    if pos <= 2 then []
    else line' :: (parse_stuff s)
  with
    Not_found -> []

let parse_answer s = 
  let refs = ref s in
  let headers = parse_stuff refs in
  let content = parse_stuff refs in
  headers,content

let request com host port =
  dprint Debug ("Connecting to "^host^":"^(string_of_int port));
  let ip_addr = Dns.get_ip host in
  let sock = ADDR_INET (ip_addr, port) in
  let conn = socket PF_INET SOCK_STREAM 0 in
  try
    (try 
      connect conn sock
    with
      Unix_error(err,s1,s2) -> failwith ("connect failed: "^(error_message err)));
    (try
      Bufrw.buf_write timeout_time conn (Bufrw.fill_buffer_from_string com)
    with
    | Unix_error(err,s1,s2) ->
	failwith ("command writing to server failed: "^s1^" "^s2^": "^(error_message err)));
    (try
      dprint Debug "send ok, getting the answer";
      let res = ref "" in
      let f s = res := !res ^ s in
      Bufrw.buf_read (Bufrw.Time timeout_time) conn f;
      dprint Debug "reading answer ok, on to parsing";
      myign Unix.close conn;
      parse_answer !res
    with
    | Unix_error(err,s1,s2) -> 
	failwith ("reading answer from server failed: "^s1^" "^s2^": "^(error_message err)));
  with
  | e -> myign Unix.close conn; raise e
