(***************************************************************)
(*                                                             *)
(*                      camlgrenouille                         *)
(*                                                             *)
(*             Copyright 2001-2003 Alan Schmitt                *)
(*              alan.schmitt@polytechnique.org                 *)
(*                                                             *)
(* This file is distributed under the terms of the GNU General *)
(* Public License.                                             *)
(*                                                             *)
(***************************************************************)

let event_list : (Event_list.event * int) list ref = ref [] 

let last_dl = ref 0

let done_successful_dl () =
  last_dl := Future.set_time 0

let time_since_last_dl () =
  (Future.set_time 0) - !last_dl

let empty_list () =
  event_list := []

let is_empty () =
  !event_list = []

let string_of_event = function
  | Event_list.Get_config -> "get config"
  | Event_list.DoDownload -> "test download"
  | Event_list.DoUpload -> "test upload"
  | Event_list.DoPing -> "test ping"
  | Event_list.DoBreakdown -> "test breakdown"
  | Event_list.PostResults -> "post results"
  | Event_list.DoDummy -> "DUMMY"

let string_of_next_event () =
  match !event_list with
  | [] -> assert false
  | (ev,t) :: el -> string_of_event ev

let rec print_event_list pf = function
  | [] -> ()
  | (ev,t) :: rest ->
    let ts = 
      let time = Unix.localtime (float_of_int t) in
      Printf.sprintf "[%04d-%02d-%02d %02d:%02d:%02d]"
      (time.Unix.tm_year + 1900)
      (time.Unix.tm_mon + 1)
      time.Unix.tm_mday
      time.Unix.tm_hour
      time.Unix.tm_min
      time.Unix.tm_sec
    in
    pf (string_of_event ev) ts; print_event_list pf rest

let time_to_next_event () =
  match !event_list with
  | [] -> assert false
  | (ev,t) :: el -> 
      let res = t - (int_of_float (Unix.time ())) in
      if res < 0 then 0 else res

let get_first_event () =
  match !event_list with
  | [] -> raise Not_found
  | (ev,t) :: el -> 
      event_list := el;
      ev

let add_event ev t =
  let rec insert_in_list ev t = function
    | [] -> [ev,t]
    | (ev', t') :: el -> 
	if t' < t then (ev',t') :: (insert_in_list ev t el)
	else (ev,t) :: ((ev',t') :: el)
  in
  event_list := insert_in_list ev t !event_list

(* We use this to make sure we only have one occurence of each event *)
let add_event_unique ev t =
  event_list := List.filter (fun (elt,_) -> ev <> elt) !event_list;
  add_event ev t

let delay_connections every_event date delay =
  Debug.dprint Debug.Debug "Event list before delay_connections";
  print_event_list
    (fun es ts -> (Debug.dprint Debug.Debug (Printf.sprintf "%s at %s" es ts)))
    !event_list;
  let rec find_events acc_found acc_other = function
  | [] -> acc_found, acc_other
  | ((ev,t) :: rest) as el ->
    if t > date then acc_found, List.rev_append acc_other el
    else if every_event then
      find_events ((ev,t + delay) :: acc_found) acc_other rest
    else begin
      match ev with
      | Event_list.Get_config
      | Event_list.PostResults ->
          find_events ((ev,t + delay) :: acc_found) acc_other rest
      | _ -> find_events acc_found ((ev,t) :: acc_other) rest
    end
  in
  let to_add, other = find_events [] [] !event_list in
  Debug.dprint Debug.Debug "Delaying the following events";
  print_event_list
    (fun es ts -> (Debug.dprint Debug.Debug (Printf.sprintf "%s at %s" es ts)))
    to_add;
  event_list := other;
  List.iter (fun (ev,t) -> add_event ev t) to_add;
  Debug.dprint Debug.Debug "Event list after delay_connections";
  print_event_list
    (fun es ts -> (Debug.dprint Debug.Debug (Printf.sprintf "%s at %s" es ts)))
    !event_list
