(***************************************************************)
(*                                                             *)
(*                 camlgrenouille for Linux                    *)
(*                                                             *)
(*             Copyright 2001-2003 Alan Schmitt                *)
(*              alan.schmitt@polytechnique.org                 *)
(*                                                             *)
(* This file is distributed under the terms of the GNU General *)
(* Public License.                                             *)
(*                                                             *)
(***************************************************************)

open Debug
open Config
open Abort

exception Error_ppp_stats_close of Unix.process_status
exception Error_plugin_info_close of Unix.process_status
exception Error_ping_close of Unix.process_status

let get_result_number res_str res_num =
  dprint Debug ("Plugin resuls is " ^ res_str ^ " and param num " ^ string_of_int(res_num));
  let res_str_2 = ref (res_str ^ " ") in
  let result_string =
    try
      for i = 1 to (res_num - 1) do
        let ind = String.index !res_str_2 ' ' in
        res_str_2 := (String.sub !res_str_2 (ind + 1) ((String.length !res_str_2) - ind - 1))
      done;
      let ind = String.index !res_str_2 ' ' in
      (String.sub !res_str_2 0 ind)
    with
    | e -> raise e
  in
  dprint Debug ("Result is " ^ result_string);
  result_string;;

let get_plugin_result plug_cmd =
  let pin = Unix.open_process_in (plug_cmd) in
  dprint Debug ("Plugin command is " ^ plug_cmd);
  let res_str =
    try
      input_line pin
    with
    | e -> myign Unix.close_process_in pin; raise e
  in
  (match Unix.close_process_in pin with
  | Unix.WEXITED 0 -> ()
  | e -> raise (Error_plugin_info_close e));
  dprint Debug ("get_plugin_result got " ^ res_str);
  res_str;;
			
let get_plugin_str () =
  let plug_str =
    let is_pass = (usrconf.plugin_password <> "") and is_user = (usrconf.plugin_username <> "") and is_port = (usrconf.plugin_port <> "") in 
    match (is_pass,is_user,is_port) with
    |(false,false,false) -> (usrconf.plugin ^ " " ^ usrconf.interface)
    |(false,false,true) -> (usrconf.plugin ^ " -h " ^ usrconf.plugin_port ^ " " ^ usrconf.interface)
    |(false,true,false) -> (usrconf.plugin ^ " -u " ^ usrconf.plugin_username ^ " " ^ usrconf.interface)
    |(false,true,true) -> (usrconf.plugin ^ " -u " ^ usrconf.plugin_username ^ " -h " ^ usrconf.plugin_port ^ " " ^ usrconf.interface)
    |(true,false,false) -> (usrconf.plugin ^ " -p " ^ usrconf.plugin_password ^ " " ^ usrconf.interface)
    |(true,false,true) -> (usrconf.plugin ^ " -p " ^ usrconf.plugin_password ^ " -h " ^ usrconf.plugin_port ^ " " ^ usrconf.interface)
    |(true,true,false) -> (usrconf.plugin ^ " -u " ^ usrconf.plugin_username ^ " -p " ^ usrconf.plugin_password ^ " " ^ usrconf.interface)
    |(true, true, true) -> (usrconf.plugin ^ " -u " ^ usrconf.plugin_username ^ " -p " ^ usrconf.plugin_password ^ " " ^ " -h " ^ usrconf.plugin_port ^ " " ^ usrconf.interface);
    in plug_str;;

let get_net_stats () =
  let plugin_str = 
    get_plugin_str()
  in
  let (r_in, r_out) =
    try
      let p_res = get_plugin_result plugin_str in
      let (res_in, res_out) = (get_result_number p_res 3, get_result_number p_res 4) in
      (float_of_string res_in, float_of_string res_out)
    with
    | e -> raise e
  in r_in, r_out;;

		
(*                
let get_net_stats () =
  let netstats = "cat /proc/net/dev " in
  let pin = Unix.open_process_in (netstats ^ " | " ^
				  usrconf.grep ^ " " ^ usrconf.interface) in
  let res_str = 
    try
      input_line pin
    with
    | e -> myign Unix.close_process_in pin; raise e
  in
  (match Unix.close_process_in pin with
  | Unix.WEXITED 0 -> ()
  | e -> raise (Error_ppp_stats_close e));
  let ind = String.index res_str ':' in
  let res_str_2 = String.sub res_str (ind + 1) ((String.length res_str) - ind - 1) in
  let ain = Unix.open_process_in ("echo \"" ^res_str_2^"\" | "^usrconf.awk ^ " '{print $1; print $9}'") in
  let res_in, res_out = 
    try
      let r_in = float_of_string (input_line ain) in
      let r_out = float_of_string (input_line ain) in
      r_in,r_out
    with
    | e -> myign Unix.close_process_in ain; raise e
  in
  myign Unix.close_process_in ain;
  res_in, res_out
*)

let ping host count ping_size =
  dprint Debug "starting ping";
  let size_string =
    if ping_size = "0" || (not usrconf.ping_string) then " " else
    " -s " ^ ping_size ^ " " in
  let pc = (usrconf.ping ^ " -q -c " ^ 
	    count ^ size_string ^ host) in
  dprint Debug pc;
  let pin = Unix.open_process_in pc in
  dprint Debug "ping process forked, waiting for output";
  let res_pack, res_trip =
    try
      let tmp_str = ref (input_line pin) in
      while not (Str.string_match (Str.regexp ".*%.*") !tmp_str 0)
      do
	dprint Debug ("Did not match:" ^ (!tmp_str));
	tmp_str := input_line pin
      done;
      let res_p = !tmp_str in
      dprint Debug res_p;
      let res_t = input_line pin in
      dprint Debug res_t;
      res_p, res_t
    with
    | e ->
	myign Unix.close_process_in pin; raise e
  in
  myign Unix.close_process_in pin;
  dprint Debug "ping acquisition ok";
  let ain = Unix.open_process_in ("echo \""^res_pack^"\" | "^usrconf.awk ^ " '{print $1; print $4}'") in
  let ping_transmitted, ping_received =
    try
      let ping_t = int_of_string (input_line ain) in
      dprint Debug ("trans: "^(string_of_int ping_t));
      let ping_r = int_of_string (input_line ain) in
      dprint Debug ("reic: "^(string_of_int ping_r));
      ping_t, ping_r
    with
    | e -> myign Unix.close_process_in ain; raise e
  in
  myign Unix.close_process_in ain;
  dprint Debug "trans reic ok";
  let cin = Unix.open_process_in ("echo \""^res_trip^"\" | "
				  ^usrconf.cut ^ " -d = -f 2 | "
				  ^usrconf.cut ^ " -d / -f 2") in
  let avg_trip =
    try
      float_of_string (input_line cin)
    with
    | e -> myign Unix.close_process_in cin; raise e
  in
  dprint Debug ("avg trip: "^(string_of_float avg_trip));
  myign Unix.close_process_in cin;
  dprint Debug "ping done";
  ping_transmitted, ping_received, ((float_of_int ping_received) *. avg_trip)

let generate_random_file filename size =
  Random.init (int_of_float (Unix.time ()));
  let out = open_out_bin filename in
  for i = 1 to size do
    output_byte out (Random.int 256)
  done;
  myign close_out out
